@@ -497,7 +497,8 @@ typedef enum { /*< flags >*/
 	CAMEL_STORE_CAN_EDIT_FOLDERS = 1 << 5,
 	CAMEL_STORE_USE_CACHE_DIR = 1 << 6,
 	CAMEL_STORE_CAN_DELETE_FOLDERS_AT_ONCE = 1 << 7,
-	CAMEL_STORE_SUPPORTS_INITIAL_SETUP = 1 << 8
+	CAMEL_STORE_SUPPORTS_INITIAL_SETUP = 1 << 8,
+	CAMEL_STORE_IS_BUILTIN = 1 << 9
 } CamelStoreFlags;
 
 /**
